# Generated by roxygen2: do not edit by hand

S3method(plot,StepReg)
S3method(print,StepReg)
export(performance)
export(report)
export(stepwise)
import(ggplot2)
importFrom(MASS,glm.nb)
importFrom(flextable,align)
importFrom(flextable,autofit)
importFrom(flextable,flextable)
importFrom(flextable,save_as_docx)
importFrom(flextable,save_as_html)
importFrom(flextable,save_as_pptx)
importFrom(flextable,save_as_rtf)
importFrom(ggrepel,geom_label_repel)
importFrom(pROC,auc)
importFrom(pROC,roc)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,reformulate)
importFrom(stats,sigma)
importFrom(stats,terms)
importFrom(survAUC,AUC.hc)
importFrom(survAUC,AUC.sh)
importFrom(survAUC,AUC.uno)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,coxph)
importFrom(utils,combn)
