---
title: "StepReg"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{StepReg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(StepReg)
```
